/*-------------------------------------------------------------------------
 *
 * pg_seclabel_d.h
 *    Macro definitions for pg_seclabel
 *
 * Portions Copyright (c) 1996-2024, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_SECLABEL_D_H
#define PG_SECLABEL_D_H

#define SecLabelRelationId 3596
#define SecLabelObjectIndexId 3597

#define Anum_pg_seclabel_objoid 1
#define Anum_pg_seclabel_classoid 2
#define Anum_pg_seclabel_objsubid 3
#define Anum_pg_seclabel_provider 4
#define Anum_pg_seclabel_label 5

#define Natts_pg_seclabel 5


#endif							/* PG_SECLABEL_D_H */
