/*-------------------------------------------------------------------------
 *
 * pg_transform_d.h
 *    Macro definitions for pg_transform
 *
 * Portions Copyright (c) 1996-2024, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_TRANSFORM_D_H
#define PG_TRANSFORM_D_H

#define TransformRelationId 3576
#define TransformOidIndexId 3574
#define TransformTypeLangIndexId 3575

#define Anum_pg_transform_oid 1
#define Anum_pg_transform_trftype 2
#define Anum_pg_transform_trflang 3
#define Anum_pg_transform_trffromsql 4
#define Anum_pg_transform_trftosql 5

#define Natts_pg_transform 5


#endif							/* PG_TRANSFORM_D_H */
