/*-------------------------------------------------------------------------
 *
 * pg_shdepend_d.h
 *    Macro definitions for pg_shdepend
 *
 * Portions Copyright (c) 1996-2024, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_SHDEPEND_D_H
#define PG_SHDEPEND_D_H

#define SharedDependRelationId 1214
#define SharedDependDependerIndexId 1232
#define SharedDependReferenceIndexId 1233

#define Anum_pg_shdepend_dbid 1
#define Anum_pg_shdepend_classid 2
#define Anum_pg_shdepend_objid 3
#define Anum_pg_shdepend_objsubid 4
#define Anum_pg_shdepend_refclassid 5
#define Anum_pg_shdepend_refobjid 6
#define Anum_pg_shdepend_deptype 7

#define Natts_pg_shdepend 7


#endif							/* PG_SHDEPEND_D_H */
